<?php

namespace Collector\Core;

use Collector\Core\Measurements\CpuLoad;
use Collector\Core\Measurements\UsedRam;
use Collector\Core\Measurements\UsedHdd;
use Collector\Core\System\Id;
use Rych\ByteSize\ByteSize;

class HealthCheck implements \JsonSerializable
{
    private $system_id;
    private $timestamp;
    private $cpu_load;
    private $used_ram;
    private $used_hdd;

    public function __construct($system_id, $cpu_load, $used_ram, $used_hdd, $timestamp = null)
    {
        $this->system_id = new Id($system_id);
        $this->timestamp = new \DateTimeImmutable($this->formatTimestamp($timestamp), new \DateTimeZone('UTC'));
        $this->cpu_load = new CpuLoad($cpu_load);
        $this->used_ram = new UsedRam($used_ram);
        $this->used_hdd = new UsedHdd($used_hdd);
    }

    public function id()
    {
        return $this->system_id->getId();
    }

    public function createdAt()
    {
        return $this->timestamp;
    }

    public function cpuLoad()
    {
        return $this->cpu_load->current();
    }

    public function usedRam()
    {
        return $this->used_ram->current();
    }

    public function usedHdd()
    {
        return $this->used_hdd->current();
    }

    public function toArray()
    {
        return [
            'id' => $this->id(),
            // Date represented in ISO 8601.
            'created_at' => $this->createdAt()->format('U'),
            'cpu_load' => $this->cpuLoad(),
            'used_ram' => $this->usedRam(),
            'used_hdd' => $this->usedHdd(),
        ];
    }

    /**
     * (PHP 5 &gt;= 5.4.0)<br/>
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     */
    public function jsonSerialize()
    {
        // Yeah, this is not the best place to set up a dependency.
        $bytes_formatter = new ByteSize();
        return [
            'id' => $this->id(),
            // Date represented in ISO 8601.
            'created_at' => $this->createdAt()->format('c'),
            'cpu_load' => number_format($this->cpuLoad(), 2),
            'used_ram' => $bytes_formatter->formatBinary($this->usedRam(), 2),
            'used_hdd' => $bytes_formatter->formatBinary($this->usedHdd(), 2),
        ];
    }

    private function formatTimestamp($timestamp)
    {
        return empty($timestamp) ? 'now' : "@$timestamp";
    }


}
